@@ -2,12 +2,16 @@
 Snap7 client used for connection to a siemens7 server.
 """
 import re
-from ctypes import c_int, c_char_p, byref, sizeof, c_uint16, c_int32, c_byte, c_void_p
+from ctypes import c_int, c_char_p, byref, sizeof, c_uint16, c_int32, c_byte
+from ctypes import c_void_p
+
 import logging
 
 import snap7
 from snap7 import six
-from snap7.snap7types import S7Object, buffer_type, buffer_size, BlocksList, param_types
+from snap7.snap7types import S7Object, buffer_type, buffer_size, BlocksList
+from snap7.snap7types import TS7BlockInfo, param_types, cpu_statuses
+
 from snap7.common import check_error, load_library, ipv4
 from snap7.snap7exceptions import Snap7Exception
 
@@ -67,6 +71,33 @@ class Client(object):
         logger.info("hot starting plc")
         return self.library.Cli_PlcColdStart(self.pointer)
 
+    def get_cpu_state(self):
+        """
+        Retrieves CPU state from client
+        """
+        state = c_int(0)
+        self.library.Cli_GetPlcStatus(self.pointer,byref(state))
+        
+        try:
+            status_string = cpu_statuses[state.value]
+        except KeyError:
+            status_string = None
+        
+        if not status_string:
+            raise Snap7Exception("The cpu state (%s) is invalid" % state.value)
+        
+        logging.debug("CPU state is %s" % status_string)
+        return status_string
+
+    def get_cpu_info(self):
+        """
+        Retrieves CPU info from client
+        """
+        info = snap7.snap7types.S7CpuInfo()
+        result = self.library.Cli_GetCpuInfo(self.pointer, byref(info))
+        check_error(result, context="client")
+        return info
+
     @error_wrap
     def disconnect(self):
         """
@@ -253,8 +284,15 @@ class Client(object):
 
     def list_blocks_of_type(self, blocktype, size):
         """This function returns the AG list of a specified block type."""
+
+        blocktype = snap7.snap7types.block_types.get(blocktype)
+
+        if not blocktype:
+            raise Snap7Exception("The blocktype parameter was invalid")
+
         logging.debug("listing blocks of type: %s size: %s" %
                       (blocktype, size))
+
         data = (c_int * 10)()
         count = c_int(size)
         result = self.library.Cli_ListBlocksOfType(
@@ -263,6 +301,26 @@ class Client(object):
             byref(count))
 
         logging.debug("number of items found: %s" % count)
+
+        check_error(result, context="client")
+        return data
+
+    def get_block_info(self, blocktype, db_number):
+        """Returns the block information for the specified block."""
+
+        blocktype = snap7.snap7types.block_types.get(blocktype)
+
+        if not blocktype:
+            raise Snap7Exception("The blocktype parameter was invalid")
+
+        logging.debug("retrieving block info for block %s of type %s" %
+                      (db_number, blocktype))
+
+        data = TS7BlockInfo()
+
+        result = self.library.Cli_GetAgBlockInfo(
+            self.pointer, blocktype,
+            db_number, byref(data))
         check_error(result, context="client")
         return data
 
@@ -485,3 +543,15 @@ class Client(object):
                                          byref(value))
         check_error(code)
         return value.value
+
+    def get_pdu_length(self):
+        """
+        Returns info about the PDU length.
+        """
+        logger.info("getting PDU length")
+        requested_ = c_uint16()
+        negotiated_ = c_uint16()
+        code = self.library.Cli_GetPduLength(self.pointer, byref(requested_), byref(negotiated_))
+        check_error(code)
+
+        return negotiated_.value
\ No newline at end of file
